@@ -14,6 +14,7 @@
 #include "soup-io-stream.h"
 #include "soup-message-queue-item.h"
 #include "soup-client-message-io-http1.h"
+#include "soup-client-message-io-http2.h"
 #include "soup-socket-properties.h"
 #include "soup-private-enum-types.h"
 #include <gio/gnetworking.h>
@@ -29,16 +30,18 @@ typedef struct {
 	SoupSocketProperties *socket_props;
         guint64 id;
         GSocketAddress *remote_address;
+        gboolean force_http1;
 
 	GUri *proxy_uri;
 	gboolean ssl;
 
-	SoupMessage *current_msg;
+	SoupMessage *proxy_msg;
         SoupClientMessageIO *io_data;
 	SoupConnectionState state;
 	time_t       unused_timeout;
 	GSource     *idle_timeout_src;
-	gboolean     reusable;
+        guint        in_use;
+        SoupHTTPVersion http_version;
 
 	GCancellable *cancellable;
 } SoupConnectionPrivate;
@@ -65,6 +68,7 @@ enum {
 	PROP_SSL,
 	PROP_TLS_CERTIFICATE,
 	PROP_TLS_CERTIFICATE_ERRORS,
+        PROP_FORCE_HTTP1,
 
 	LAST_PROPERTY
 };
@@ -81,6 +85,9 @@ static void stop_idle_timer (SoupConnectionPrivate *priv);
 static void
 soup_connection_init (SoupConnection *conn)
 {
+        SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
+
+        priv->http_version = SOUP_HTTP_1_1;
 }
 
 static void
@@ -93,7 +100,7 @@ soup_connection_finalize (GObject *object)
         g_clear_pointer (&priv->io_data, soup_client_message_io_destroy);
 	g_clear_object (&priv->remote_connectable);
         g_clear_object (&priv->remote_address);
-	g_clear_object (&priv->current_msg);
+	g_clear_object (&priv->proxy_msg);
 
 	if (priv->cancellable) {
 		g_warning ("Disposing connection %p during connect", object);
@@ -135,15 +142,15 @@ soup_connection_set_property (GObject *object, guint prop_id,
 	case PROP_SOCKET_PROPERTIES:
 		priv->socket_props = g_value_dup_boxed (value);
 		break;
-	case PROP_STATE:
-		soup_connection_set_state (SOUP_CONNECTION (object), g_value_get_uint (value));
-		break;
 	case PROP_SSL:
 		priv->ssl = g_value_get_boolean (value);
 		break;
 	case PROP_ID:
 		priv->id = g_value_get_uint64 (value);
 		break;
+	case PROP_FORCE_HTTP1:
+		priv->force_http1 = g_value_get_boolean (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -181,6 +188,9 @@ soup_connection_get_property (GObject *object, guint prop_id,
 	case PROP_TLS_CERTIFICATE_ERRORS:
 		g_value_set_flags (value, soup_connection_get_tls_certificate_errors (SOUP_CONNECTION (object)));
 		break;
+	case PROP_FORCE_HTTP1:
+		g_value_set_boolean (value, priv->force_http1);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -254,8 +264,9 @@ soup_connection_class_init (SoupConnectionClass *connection_class)
 		g_param_spec_enum ("state",
 				   "Connection state",
 				   "Current state of connection",
-				   SOUP_TYPE_CONNECTION_STATE, SOUP_CONNECTION_NEW,
-				   G_PARAM_READWRITE |
+				   SOUP_TYPE_CONNECTION_STATE,
+                                   SOUP_CONNECTION_NEW,
+				   G_PARAM_READABLE |
 				   G_PARAM_STATIC_STRINGS);
         properties[PROP_SSL] =
 		g_param_spec_boolean ("ssl",
@@ -284,6 +295,12 @@ soup_connection_class_init (SoupConnectionClass *connection_class)
                                     G_TYPE_TLS_CERTIFICATE_FLAGS, 0,
                                     G_PARAM_READABLE |
                                     G_PARAM_STATIC_STRINGS);
+        properties[PROP_FORCE_HTTP1] =
+                g_param_spec_boolean ("force-http1",
+                                      "Force HTTP 1.x",
+                                      "Force connection to use HTTP 1.x",
+                                      FALSE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+                                      G_PARAM_STATIC_STRINGS);
 
         g_object_class_install_properties (object_class, LAST_PROPERTY, properties);
 }
@@ -329,64 +346,64 @@ stop_idle_timer (SoupConnectionPrivate *priv)
 }
 
 static void
-current_msg_got_body (SoupMessage *msg, gpointer user_data)
+soup_connection_set_state (SoupConnection     *conn,
+                           SoupConnectionState state)
+{
+        SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
+
+        if (priv->state == state)
+                return;
+
+        priv->state = state;
+        if (priv->state == SOUP_CONNECTION_IDLE)
+                start_idle_timer (conn);
+
+        g_object_notify_by_pspec (G_OBJECT (conn), properties[PROP_STATE]);
+}
+
+static void
+proxy_msg_got_body (SoupMessage    *msg,
+                    SoupConnection *conn)
 {
-	SoupConnection *conn = user_data;
 	SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
 
-	priv->unused_timeout = 0;
-
-	if (priv->proxy_uri &&
-	    soup_message_get_method (msg) == SOUP_METHOD_CONNECT &&
-	    SOUP_STATUS_IS_SUCCESSFUL (soup_message_get_status (msg))) {
+	if (SOUP_STATUS_IS_SUCCESSFUL (soup_message_get_status (msg))) {
 		soup_connection_event (conn, G_SOCKET_CLIENT_PROXY_NEGOTIATED, NULL);
 
 		/* We're now effectively no longer proxying */
 		g_clear_pointer (&priv->proxy_uri, g_uri_unref);
+                g_signal_handlers_disconnect_by_func (priv->proxy_msg, proxy_msg_got_body, conn);
+                g_clear_object (&priv->proxy_msg);
 	}
-
-	priv->reusable = soup_message_is_keepalive (msg);
 }
 
 static void
-clear_current_msg (SoupConnection *conn)
+clear_proxy_msg (SoupConnection *conn)
 {
 	SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
-	SoupMessage *msg;
 
-	msg = priv->current_msg;
-	priv->current_msg = NULL;
+        if (!priv->proxy_msg)
+                return;
 
-	g_signal_handlers_disconnect_by_func (msg, G_CALLBACK (current_msg_got_body), conn);
-	g_object_unref (msg);
+        g_signal_handlers_disconnect_by_func (priv->proxy_msg, proxy_msg_got_body, conn);
+        g_clear_object (&priv->proxy_msg);
 }
 
 static void
-set_current_msg (SoupConnection *conn, SoupMessage *msg)
+set_proxy_msg (SoupConnection *conn,
+               SoupMessage    *msg)
 {
 	SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
 
-	g_return_if_fail (priv->state == SOUP_CONNECTION_IN_USE);
+        g_assert (priv->http_version != SOUP_HTTP_2_0);
 
-	g_object_freeze_notify (G_OBJECT (conn));
+        clear_proxy_msg (conn);
+	priv->proxy_msg = g_object_ref (msg);
+	g_signal_connect_object (msg, "got-body",
+                                 G_CALLBACK (proxy_msg_got_body),
+                                 conn, 0);
 
-	if (priv->current_msg) {
-		g_return_if_fail (soup_message_get_method (priv->current_msg) == SOUP_METHOD_CONNECT);
-		clear_current_msg (conn);
-	}
-
-	stop_idle_timer (priv);
-
-	priv->current_msg = g_object_ref (msg);
-	priv->reusable = FALSE;
-
-	g_signal_connect (msg, "got-body",
-			  G_CALLBACK (current_msg_got_body), conn);
-
-	if (priv->proxy_uri && soup_message_get_method (msg) == SOUP_METHOD_CONNECT)
-		soup_connection_event (conn, G_SOCKET_CLIENT_PROXY_NEGOTIATING, NULL);
-
-	g_object_thaw_notify (G_OBJECT (conn));
+        soup_connection_event (conn, G_SOCKET_CLIENT_PROXY_NEGOTIATING, NULL);
 }
 
 static void
@@ -395,12 +412,31 @@ soup_connection_set_connection (SoupConnection *conn,
 {
 	SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
 
+        g_clear_pointer (&priv->io_data, soup_client_message_io_destroy);
+
 	g_clear_object (&priv->connection);
 	priv->connection = connection;
 	g_clear_object (&priv->iostream);
 	priv->iostream = soup_io_stream_new (G_IO_STREAM (priv->connection), FALSE);
 }
 
+static void
+soup_connection_create_io_data (SoupConnection *conn)
+{
+        SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
+
+        g_assert (!priv->io_data);
+        switch (priv->http_version) {
+        case SOUP_HTTP_1_0:
+        case SOUP_HTTP_1_1:
+                priv->io_data = soup_client_message_io_http1_new (priv->iostream);
+                break;
+        case SOUP_HTTP_2_0:
+                priv->io_data = soup_client_message_io_http2_new (priv->iostream, priv->id);
+                break;
+        }
+}
+
 static void
 re_emit_socket_event (GSocketClient       *client,
 		      GSocketClientEvent   event,
@@ -467,6 +503,15 @@ new_tls_connection (SoupConnection    *conn,
 {
         SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
         GTlsClientConnection *tls_connection;
+        GPtrArray *advertised_protocols = g_ptr_array_sized_new (4);
+
+        // https://www.iana.org/assignments/tls-extensiontype-values/tls-extensiontype-values.xhtml
+        if (!priv->force_http1)
+                g_ptr_array_add (advertised_protocols, "h2");
+
+        g_ptr_array_add (advertised_protocols, "http/1.1");
+        g_ptr_array_add (advertised_protocols, "http/1.0");
+        g_ptr_array_add (advertised_protocols, NULL);
 
         tls_connection = g_initable_new (g_tls_backend_get_client_connection_type (g_tls_backend_get_default ()),
                                          priv->cancellable, error,
@@ -474,7 +519,11 @@ new_tls_connection (SoupConnection    *conn,
                                          "server-identity", priv->remote_connectable,
                                          "require-close-notify", FALSE,
                                          "interaction", priv->socket_props->tls_interaction,
+                                         "advertised-protocols", advertised_protocols->pdata,
                                          NULL);
+
+        g_ptr_array_unref (advertised_protocols);
+
         if (!tls_connection)
                 return NULL;
 
@@ -543,12 +592,24 @@ soup_connection_complete (SoupConnection *conn)
 
         g_clear_object (&priv->cancellable);
 
+        if (G_IS_TLS_CONNECTION (priv->connection)) {
+                const char *protocol = g_tls_connection_get_negotiated_protocol (G_TLS_CONNECTION (priv->connection));
+                if (g_strcmp0 (protocol, "h2") == 0)
+                        priv->http_version = SOUP_HTTP_2_0;
+                else if (g_strcmp0 (protocol, "http/1.0") == 0)
+                        priv->http_version = SOUP_HTTP_1_0;
+                else if (g_strcmp0 (protocol, "http/1.1") == 0)
+                        priv->http_version = SOUP_HTTP_1_1;
+        }
+
         if (!priv->ssl || !priv->proxy_uri) {
                 soup_connection_event (conn,
                                        G_SOCKET_CLIENT_COMPLETE,
                                        NULL);
         }
 
+        soup_connection_create_io_data (conn);
+
         soup_connection_set_state (conn, SOUP_CONNECTION_IN_USE);
         priv->unused_timeout = time (NULL) + SOUP_CONNECTION_UNUSED_TIMEOUT;
         start_idle_timer (conn);
@@ -729,6 +790,10 @@ tunnel_handshake_ready_cb (GTlsConnection *tls_connection,
         if (g_tls_connection_handshake_finish (tls_connection, result, &error)) {
                 soup_connection_event (conn, G_SOCKET_CLIENT_TLS_HANDSHAKED, NULL);
                 soup_connection_event (conn, G_SOCKET_CLIENT_COMPLETE, NULL);
+
+                g_assert (!priv->io_data);
+                priv->io_data = soup_client_message_io_http1_new (priv->iostream);
+
                 g_task_return_boolean (task, TRUE);
         } else {
                 g_task_return_error (task, error);
@@ -817,6 +882,9 @@ soup_connection_tunnel_handshake (SoupConnection *conn,
         soup_connection_event (conn, G_SOCKET_CLIENT_TLS_HANDSHAKED, NULL);
         soup_connection_event (conn, G_SOCKET_CLIENT_COMPLETE, NULL);
 
+        g_assert (!priv->io_data);
+        priv->io_data = soup_client_message_io_http1_new (priv->iostream);
+
         return TRUE;
 }
 
@@ -837,8 +905,7 @@ soup_connection_disconnect (SoupConnection *conn)
         priv = soup_connection_get_instance_private (conn);
 
         old_state = priv->state;
-        if (old_state != SOUP_CONNECTION_DISCONNECTED)
-                soup_connection_set_state (conn, SOUP_CONNECTION_DISCONNECTED);
+        soup_connection_set_state (conn, SOUP_CONNECTION_DISCONNECTED);
 
         if (priv->cancellable) {
                 g_cancellable_cancel (priv->cancellable);
@@ -933,115 +1000,55 @@ soup_connection_is_via_proxy (SoupConnection *conn)
 	return priv->proxy_uri != NULL;
 }
 
-static gboolean
-is_idle_connection_disconnected (SoupConnection *conn)
+gboolean
+soup_connection_is_idle_open (SoupConnection *conn)
 {
-	SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
-	GInputStream *istream;
-	char buffer[1];
-	GError *error = NULL;
+        SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
+
+        g_assert (priv->state == SOUP_CONNECTION_IDLE);
 
 	if (!g_socket_is_connected (soup_connection_get_socket (conn)))
-		return TRUE;
+		return FALSE;
 
 	if (priv->unused_timeout && priv->unused_timeout < time (NULL))
-		return TRUE;
+		return FALSE;
 
-	istream = g_io_stream_get_input_stream (priv->iostream);
-
-	/* This is tricky. The goal is to check if the socket is readable. If
-	 * so, that means either the server has disconnected or it's broken (it
-	 * should not send any data while the connection is in idle state). But
-	 * we can't just check the readability of the SoupSocket because there
-	 * could be non-application layer TLS data that is readable, but which
-	 * we don't want to consider. So instead, just read and see if the read
-	 * succeeds. This is OK to do here because if the read does succeed, we
-	 * just disconnect and ignore the data anyway.
-	 */
-	g_pollable_input_stream_read_nonblocking (G_POLLABLE_INPUT_STREAM (istream),
-						  &buffer, sizeof (buffer),
-						  NULL, &error);
-	if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
-		g_clear_error (&error);
-		return TRUE;
-	}
-
-	g_error_free (error);
-
-	return FALSE;
+        return soup_client_message_io_is_open (priv->io_data);
 }
 
 SoupConnectionState
 soup_connection_get_state (SoupConnection *conn)
 {
-	SoupConnectionPrivate *priv;
-
-	g_return_val_if_fail (SOUP_IS_CONNECTION (conn),
-			      SOUP_CONNECTION_DISCONNECTED);
-	priv = soup_connection_get_instance_private (conn);
-
-	if (priv->state == SOUP_CONNECTION_IDLE &&
-	    is_idle_connection_disconnected (conn))
-		soup_connection_set_state (conn, SOUP_CONNECTION_REMOTE_DISCONNECTED);
+	SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
 
 	return priv->state;
 }
 
 void
-soup_connection_set_state (SoupConnection *conn, SoupConnectionState state)
-{
-	SoupConnectionPrivate *priv;
-
-	g_return_if_fail (SOUP_IS_CONNECTION (conn));
-	g_return_if_fail (state >= SOUP_CONNECTION_NEW &&
-			  state <= SOUP_CONNECTION_DISCONNECTED);
-
-        g_object_ref (conn);
-	g_object_freeze_notify (G_OBJECT (conn));
-
-	priv = soup_connection_get_instance_private (conn);
-
-	if (priv->current_msg) {
-		g_warn_if_fail (state == SOUP_CONNECTION_IDLE ||
-				state == SOUP_CONNECTION_DISCONNECTED);
-		clear_current_msg (conn);
-	}
-
-	if (state == SOUP_CONNECTION_IDLE && !priv->reusable) {
-		/* This will recursively call set_state() */
-		soup_connection_disconnect (conn);
-	} else {
-		priv->state = state;
-
-		if (priv->state == SOUP_CONNECTION_IDLE)
-			start_idle_timer (conn);
-
-		g_object_notify_by_pspec (G_OBJECT (conn), properties[PROP_STATE]);
-	}
-
-	g_object_thaw_notify (G_OBJECT (conn));
-        g_object_unref (conn);
-}
-
-void
-soup_connection_set_reusable (SoupConnection *conn,
-                              gboolean        reusable)
+soup_connection_set_in_use (SoupConnection *conn,
+                            gboolean        in_use)
 {
         SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
 
-        g_return_if_fail (SOUP_IS_CONNECTION (conn));
+        g_assert (in_use || priv->in_use > 0);
 
-        priv->reusable = TRUE;
-}
+        if (in_use)
+                priv->in_use++;
+        else
+                priv->in_use--;
 
-gboolean
-soup_connection_get_ever_used (SoupConnection *conn)
-{
-	SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
+        if (priv->in_use > 0) {
+                if (priv->state == SOUP_CONNECTION_IDLE)
+                        soup_connection_set_state (conn, SOUP_CONNECTION_IN_USE);
+                return;
+        }
 
-	g_return_val_if_fail (SOUP_IS_CONNECTION (conn), FALSE);
+        clear_proxy_msg (conn);
 
-	return priv->unused_timeout == 0;
+        if (soup_connection_is_reusable (conn))
+                soup_connection_set_state (conn, SOUP_CONNECTION_IDLE);
+        else
+                soup_connection_disconnect (conn);
 }
 
 SoupClientMessageIO *
@@ -1050,30 +1057,22 @@ soup_connection_setup_message_io (SoupConnection *conn,
 {
         SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
 
-        g_assert (priv->state != SOUP_CONNECTION_NEW &&
-                  priv->state != SOUP_CONNECTION_DISCONNECTED);
+        g_assert (priv->state == SOUP_CONNECTION_IN_USE);
 
-        if (priv->current_msg != msg)
-                set_current_msg (conn, msg);
-        else
-                priv->reusable = FALSE;
+        priv->unused_timeout = 0;
+        stop_idle_timer (priv);
 
-        g_assert (priv->io_data == NULL);
-        priv->io_data = soup_client_message_io_http1_new (priv->iostream);
+        if (priv->proxy_uri && soup_message_get_method (msg) == SOUP_METHOD_CONNECT)
+                set_proxy_msg (conn, msg);
+
+        if (!soup_client_message_io_is_reusable (priv->io_data)) {
+                g_clear_pointer (&priv->io_data, soup_client_message_io_destroy);
+                soup_connection_create_io_data (conn);
+        }
 
         return priv->io_data;
 }
 
-void
-soup_connection_message_io_finished (SoupConnection *conn,
-                                     SoupMessage    *msg)
-{
-        SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
-
-        g_assert (priv->current_msg == msg);
-        g_clear_pointer (&priv->io_data, soup_client_message_io_destroy);
-}
-
 GTlsCertificate *
 soup_connection_get_tls_certificate (SoupConnection *conn)
 {
@@ -1119,3 +1118,19 @@ soup_connection_get_remote_address (SoupConnection *conn)
 
         return priv->remote_address;
 }
+
+SoupHTTPVersion
+soup_connection_get_negotiated_protocol (SoupConnection *conn)
+{
+        SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
+
+        return priv->http_version;
+}
+
+gboolean
+soup_connection_is_reusable (SoupConnection *conn)
+{
+        SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
+
+        return priv->io_data && soup_client_message_io_is_reusable (priv->io_data);
+}
