@@ -46,8 +46,12 @@ enum {
 	PROP_0,
 
 	PROP_MESSAGE,
+
+        LAST_PROPERTY
 };
 
+static GParamSpec *properties[LAST_PROPERTY] = { NULL, };
+
 struct _SoupMultipartInputStream {
 	GFilterInputStream parent_instance;
 };
@@ -306,14 +310,16 @@ soup_multipart_input_stream_class_init (SoupMultipartInputStreamClass *multipart
 
 	input_stream_class->read_fn = soup_multipart_input_stream_read;
 
-	g_object_class_install_property (
-		object_class, PROP_MESSAGE,
+        properties[PROP_MESSAGE] =
 		g_param_spec_object ("message",
 				     "Message",
 				     "The SoupMessage",
 				     SOUP_TYPE_MESSAGE,
-				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-				     G_PARAM_STATIC_STRINGS));
+				     G_PARAM_READWRITE |
+                                     G_PARAM_CONSTRUCT_ONLY |
+				     G_PARAM_STATIC_STRINGS);
+
+        g_object_class_install_properties (object_class, LAST_PROPERTY, properties);
 
 }
 
