@@ -7,6 +7,7 @@
 #define __SOUP_SESSION_PRIVATE_H__ 1
 
 #include "soup-session.h"
+#include "soup-content-processor.h"
 
 G_BEGIN_DECLS
 
@@ -22,6 +23,11 @@ void     soup_session_cancel_message        (SoupSession *session,
 SoupMessage *soup_session_get_original_message_for_authentication (SoupSession *session,
 								   SoupMessage *msg);
 
+GInputStream *soup_session_setup_message_body_input_stream (SoupSession        *session,
+                                                            SoupMessage        *msg,
+                                                            GInputStream       *body_stream,
+                                                            SoupProcessingStage start_at_stage);
+
 G_END_DECLS
 
 #endif /* __SOUP_SESSION_PRIVATE_H__ */
